#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typedef.h"

class Link {
public:
  int nodenr;
  long weight;
  Link *nextptr;
};

class ProblemClass {
public:
  int nrofnodes;
  char ProblemFilename[MAXPATHLENGTH];
  Link* gradjl[MAXGENES];            /* edges of graph in adjacency list */
  int nrofneighbors[MAXGENES];
  long weight[MAXGENES];
  void CreateProblem(int, double, int);
  void RemoveProblem();
  void ReadProblem(int&);  
  void DeleteAdjlist();
  void PrintAdjList();
};


void ProblemClass::DeleteAdjlist() {

  int i;
  Link *temp;

  for (i = 0; i < nrofnodes; i++)
    while (gradjl[i]) {
      temp = gradjl[i]->nextptr;
      delete gradjl[i];
      gradjl[i] = temp;
    }

} /* ProblemClass::DeleteAdjlist() */


void ProblemClass::PrintAdjList() {
  
  int i;
  Link *temp;
 
  for (i = 0; i < nrofnodes; i++) {
    cout << "buren van node " << i << ": ";
    temp = gradjl[i];
    while (temp) {
      cout << temp->nodenr << " ";
      temp = temp->nextptr;
    }
    cout << endl;
  }

} /* ProblemClass::DeleteAdjList() */

void ProblemClass::CreateProblem(int n, double p, int s) {
  
  int i, l;
  char string[MAXPATHLENGTH];
  ofstream output;
  const int DECIMALS = 3;
    
  strcpy(ProblemFilename, PROBLEMNAME);
  l = strlen(ProblemFilename);
  ProblemFilename[l++] = 'n';
  for (i = (int) log10((double) n); i >= 0; i--) 
    ProblemFilename[l++] = '0' + (n % ((int) pow(10,i+1))) / ((int) pow(10, i));
  ProblemFilename[l++] = 'p';
  if (p < 1.0) 
    ProblemFilename[l++] = '0';
  else
    ProblemFilename[l++] = '1';
  ProblemFilename[l++] = '.';

  for (i = 1; i <= DECIMALS; i++) 
    ProblemFilename[l++] = '0' + ((int)(pow(10, i)*p+pow(10,-DECIMALS-1))) % 10;
  ProblemFilename[l++] = 's';
  ProblemFilename[l++] = '0' + s / 10;
  ProblemFilename[l++] = '0' + s % 10;
  ProblemFilename[l++] = (char) NULL;

  strcpy(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  strcat(string,"ingen");

  output.open(string);
  output << 0 << endl;      // ascii code
  output << s << endl;      // seednr s
  output << 2 << endl;      // equipartite
  output << n << endl;      // nr of nodes
  output << 3 << endl;      // 3-colorings
  output << 1 << endl;      // IID-edge probability
  output << p << endl;      // edge probabilty p
  output << 0 << endl;      // no cheats
  output.close();

  strcpy(string, "generator ");
  strcat(string, PROBLEMDIR);
  strcat (string, ProblemFilename);
  strcat(string, " < ");
  strcat(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  strcat(string, "ingen > ");
  strcat(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  strcat(string, "NULL");
  system(string); 

} /* ProblemClass::CreateProblem() */

void ProblemClass::RemoveProblem() {

  char string[MAXPATHLENGTH];

  strcpy(string, "rm ");
  strcat(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  system(string);
  strcpy(string, "rm ");
  strcat(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  strcat(string, "ingen");
  system(string);

  strcpy(string, "rm ");
  strcat(string, PROBLEMDIR);
  strcat(string, ProblemFilename);
  strcat(string, "NULL");
  system(string);

  DeleteAdjlist();

} /* ProblemClass::RemoveProblem() */


void ProblemClass::ReadProblem(int &nrofgenes) {

  const int MAXLEN = 250;
  ifstream input;
  int n1, n2, i;
  char kar, string[MAXLEN];
  Link  *temp;

  strcpy (string, PROBLEMDIR);
  strcat (string, ProblemFilename);
  input.open(string);

  cout << "# Problem \""<< string << "\" ";
  /* Find line starting with p, which includes the nr of nodes */
  input >> kar;
  while (kar != 'p') {
    input.getline(string, MAXLEN);
    input >> kar;
  }
  
  input >> string;
  input >> nrofnodes;
  nrofgenes = nrofnodes;
  input.getline(string, MAXLEN);

  /* Initialize edges of graph at non-existent */
  for (i = 0; i < nrofgenes; i++) {
    gradjl[i] = NULL;
    nrofneighbors[i] = 0;
    weight[i] = 1;
  }
  
  /* Find first line starting with an 'e' to get the edges */ 
  input >> kar;
  while (kar != 'e') {
    input.getline(string, MAXLEN);
    input >> kar;
  }
  while (!input.eof()) {
    input >> n1 >> n2;
    // cout << "Edge (" << n1 << "," << n2 << ")" << endl; 
    temp = gradjl[n1-1];
    gradjl[n1-1] = new Link;
    gradjl[n1-1]->nodenr = n2-1;
    gradjl[n1-1]->weight = 1;
    gradjl[n1-1]->nextptr = temp;
    nrofneighbors[n1-1]++;
    temp = gradjl[n2-1];
    gradjl[n2-1] = new Link;
    gradjl[n2-1]->nodenr = n1-1;
    gradjl[n2-1]->weight = 1;
    gradjl[n2-1]->nextptr = temp;
    nrofneighbors[n2-1]++;
    input >> kar;
  }

  input.close();

  cout << "read." << endl;
  cout << "# " << nrofgenes << " nodes" << endl;
} /* ProblemClass::ReadProblem() */

ProblemClass problem;
